package com.loit.Proxy;

import org.springframework.beans.factory.FactoryBean;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Proxy;

/**
 * 接口实例工厂，这里主要是用于提供接口的实例对象
 *
 * @author Demon add by 2020-05-28
 */
public class ApiProxyBeanFactory<T> implements FactoryBean<T> {
 
    private Class<T> interfaceType;
 
    public ApiProxyBeanFactory(Class<T> interfaceType) {
        this.interfaceType = interfaceType;
    }
 
    @Override
    public T getObject() throws Exception {
        //这里主要是创建接口对应的实例，便于注入到spring容器中
        InvocationHandler handler = new ApiProxyHandler<>(interfaceType);
        return (T) Proxy.newProxyInstance(interfaceType.getClassLoader(), new Class[] {interfaceType},handler);
    }
 
    @Override
    public Class<T> getObjectType() {
        return interfaceType;
    }
 
    @Override
    public boolean isSingleton() {
        return true;
    }
}
