package com.loit.api.service;

import com.loit.api.dto.ContextConfigurationDTO;
import com.loit.common.json.AjaxJson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.*;


/**
 * @description 上下文配置
 * @author xutao
 * @date 2019-10-18
 */
@FeignClient(name = "loit-portal")
public interface ContextConfigurationApiService {

    @GetMapping(value = "api/v1/feign/contextConfig/list")
    @ApiOperation(value="列表", notes="获取上下文配置列表")
    public AjaxJson list();

    @GetMapping(value = "api/v1/feign/contextConfig/page")
    @ApiOperation(value="分页列表", notes="获取上下文配置分页列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNo", value = "当前页码", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "分页大小", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "contextValue", value = "上下文值", required = false, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "contextDesc", value = "描述", required = false, dataType = "String", paramType = "query")})
    public AjaxJson page(
            @RequestParam(value = "pageNo", required = true) Integer pageNo,
            @RequestParam(value = "pageSize", required = true) Integer pageSize,
            @RequestParam(value = "contextValue", required = false) String contextValue,
            @RequestParam(value = "contextDesc", required = false) String contextDesc);

    @GetMapping(value = "api/v1/feign/contextConfig/{contextId}")
    @ApiOperation(value="详情", notes="根据contextId获取上下文配置详情")
    @ApiImplicitParam(name = "contextId", value = "上下文配置ID", required = true, dataType = "String")
    public AjaxJson get(@PathVariable(required=false) String contextId);

    @PostMapping(value = "api/v1/feign/contextConfig/save")
    @ApiOperation(value = "保存", notes = "新增上下文配置")
    @ApiImplicitParam(name = "contextConfigurationDTO", value = "上下文配置数据传输实体类", required = true, dataType = "ContextConfigurationDTO")
    public AjaxJson save(@RequestBody ContextConfigurationDTO contextConfigurationDTO);

    @PostMapping(value = "api/v1/feign/contextConfig/{contextId}")
    @ApiOperation(value = "修改", notes = "修改上下文配置")
    public AjaxJson update(@PathVariable int contextId,@RequestBody ContextConfigurationDTO contextConfigurationDTO);

    @ApiOperation(value = "删除", notes = "单个或批量删除上下文配置，多ID时，以逗号分隔")
    @DeleteMapping(value = "api/v1/feign/contextConfig/{contextId}")
    public AjaxJson deleteAll(@PathVariable("contextId")String ids);
    
    @GetMapping(value = "api/v1/feign/contextConfig/serviceId/{serviceId}")
    public AjaxJson getByserviceId(@PathVariable("contextId")String ids);

    @ApiOperation(value = "检查上下文描述是否重复", notes = "检查上下文描述是否重复")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "contextId", value = "上下文ID", required = false, dataType = "int", paramType = "query"),
            @ApiImplicitParam(name = "contextDesc", value = "上下文描述", required = true, dataType = "String", paramType = "query")})
    @GetMapping(value = "api/v1/feign/contextConfig/checkDescUnique")
    public AjaxJson checkDictNameUnique(
            @RequestParam(value = "contextId", required = false) Integer contextId,
            @RequestParam(value = "contextDesc", required = true) String contextDesc);
}
