package com.loit.api.service;

import com.loit.common.json.AjaxJson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.*;

/**
 * @description 系统字典数据项Controller
 * @author xutao
 * @date 2019-10-18
 */
@FeignClient(name = "loit-portal")
public interface DictDataApiService {
	/**
	 * @description 系统字典数据项分页列表
	 * @author xutao
	 * @date：2019年10月18日
	 * @return
	 */
	@GetMapping(value = "api/v1/feign/dictData/page")
	@ApiOperation(value="分页列表", notes="获取系统字典数据项分页列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNo", value = "当前页码", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "分页大小", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "dictId", value = "字典ID", required = true, dataType = "String", paramType = "query")		
	})	
	public AjaxJson page(
			@RequestParam(value = "pageNo", required = true) Integer pageNo,
			@RequestParam(value = "pageSize", required = true) Integer pageSize,
			@RequestParam(value = "dictId", required = true) Integer dictId);
	
	/**
	 * @description 系统字典数据项列表
	 * @author xutao
	 * @date：2019年10月18日
	 * @return
	 */
	@GetMapping(value = "api/v1/feign/dictData/list")
	@ApiOperation(value="列表", notes="获取系统字典数据项列表")
	@ApiImplicitParam(name = "dictId", value = "字典ID", required = true, dataType = "String", paramType = "query")
	public AjaxJson list(@RequestParam(value = "dictId", required = true) Integer dictId);

	/**
	 * @description 系统字典数据项详情
	 * @author xutao
	 * @date：2019年10月18日
	 * @param dataId
	 * @return
	 */
	@GetMapping(value = "api/v1/feign/dictData/{dataId}")
	@ApiOperation(value="详情", notes="根据dataId获取系统字典数据项详情")
	@ApiImplicitParam(name = "dataId", value = "系统字典数据项ID", required = true, dataType = "String")
	public AjaxJson get(@PathVariable(required=true) String dataId);

	/**
	 * @description 从缓存中获取某字典数据项列表，如果缓存中没有，则从数据库中获取
	 * @return
	 */
	@GetMapping(value = "api/v1/feign/dictData/getDictDataByDictId")
	@ApiOperation(value="从缓存中获取某字典数据项列表", notes="从缓存中获取某字典数据项列表，如果缓存中没有，则从数据库中获取")
	@ApiImplicitParam(name = "dictCode", value = "字典dictCode", required = true, dataType = "String", paramType="query")
	public AjaxJson listDictDataByDictId(@RequestParam(value = "dictCode", required = true) String dictCode);
	
	/**
	 * @description 检查字典项名称是否重复
	 * @return
	 */
	@ApiOperation(value = "检查字典项名称是否重复", notes = "检查字典项名称是否重复")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "dictId", value = "字典ID", required = true, dataType = "int", paramType = "query"),
		@ApiImplicitParam(name = "dataId", value = "字典项ID", required = false, dataType = "int", paramType = "query"),
		@ApiImplicitParam(name = "dataName", value = "字典项名称", required = true, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "api/v1/feign/dictData/checkDataNameUnique")
	public AjaxJson checkDataNameUnique(
			@RequestParam(value = "dictId", required = true) Integer dictId,
			@RequestParam(value = "dataId", required = false) Integer dataId,
			@RequestParam(value = "dataName", required = true) String dataName);

}
