package com.loit.api.service;

import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import com.loit.api.dto.Org;
import com.loit.common.json.AjaxJson;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**  
* @ClassName: OrgController  
* @Description:   
* @author zhaokz  
* @date 2019年10月22日 下午5:41:41 
* @version V1.0  
*/
@FeignClient(name = "loit-portal")
public interface OrgApiService {

	/**
	 * 获取组织树形列表
	 * @return
	 */
	@ApiOperation(value = "获取组织树形列表", notes = "获取组织树形列表")
	@GetMapping(value = "api/v1/feign/org/treeList")
	public AjaxJson treeList();

	/**
	 * 获取当前应用组织树形列表
	 * @return
	 */
	@ApiOperation(value = "获取当前应用组织树形列表", notes = "获取当前应用组织树形列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "appId", value = "应用id", required = true, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "api/v1/feign/org/currentApp/treeList1")
	public AjaxJson treeList1(
			@RequestParam(value = "appId", required = true) String appId);
	
	/**
	 * @description 根据组织编码获取当前应用组织树形列表
	 * @return
	 */
	@ApiOperation(value = "根据组织编码获取当前应用组织树形列表", notes = "")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "appId", value = "应用id", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "orgCode", value = "组织编码", required = true, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "api/v1/feign/org/currentApp/treeList2")
	public AjaxJson treeList2(
			@RequestParam(value = "appId", required = true) String appId,
			@RequestParam(value = "orgCode", required = true) String orgCode);

	/**
	 * 获取组织分页列表
	 * @return
	 */
	@ApiOperation(value = "获取组织分页列表", notes = "获取组织分页列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNo", value = "当前页码", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "分页大小", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "appId", value = "应用id", required = false, dataType = "String", paramType = "query") ,
		@ApiImplicitParam(name = "orgId", value = "组织ID", required = false, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "orgName", value = "组织名称", required = false, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "orgCode", value = "组织编码", required = false, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "deltag", value = "删除标记", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "queryLabelName", value = "标签名称", required = false, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "api/v1/feign/org/pageList")
	public AjaxJson list(
			@RequestParam(value = "pageNo", required = true) Integer pageNo,
			@RequestParam(value = "pageSize", required = true) Integer pageSize,
			@RequestParam(value = "appId", required = false) String appId,
			@RequestParam(value = "orgId", required = false) String orgId,
			@RequestParam(value = "orgName", required = false) String orgName,
			@RequestParam(value = "orgCode", required = false) String orgCode,
			@RequestParam(value = "deltag", required = true) String deltag,
			@RequestParam(value = "queryLabelName", required = false) String queryLabelName);

	/**
	 * 获取组织分页列表
	 * @return
	 */
	@ApiOperation(value = "获取组织分页列表-无登录用户", notes = "获取组织分页列表-无登录用户")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "pageNo", value = "当前页码", required = true, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "pageSize", value = "分页大小", required = true, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "appId", value = "应用id", required = false, dataType = "String", paramType = "query") ,
			@ApiImplicitParam(name = "orgId", value = "组织ID", required = false, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "orgName", value = "组织名称", required = false, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "orgCode", value = "组织编码", required = false, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "deltag", value = "删除标记", required = true, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "queryLabelName", value = "标签名称", required = false, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "api/v1/feign/org/nouser/pageList")
	public AjaxJson noUserList(
			@RequestParam(value = "pageNo", required = true) Integer pageNo,
			@RequestParam(value = "pageSize", required = true) Integer pageSize,
			@RequestParam(value = "appId", required = false) String appId,
			@RequestParam(value = "orgId", required = false) String orgId,
			@RequestParam(value = "orgName", required = false) String orgName,
			@RequestParam(value = "orgCode", required = false) String orgCode,
			@RequestParam(value = "deltag", required = true) String deltag,
			@RequestParam(value = "queryLabelName", required = false) String queryLabelName);

	/**
	 * 获取组织详细信息
	 * @param orgId
	 * @return
	 */
	@ApiOperation(value = "获取组织详细信息", notes = "根据id来获取Org详细信息")
	@ApiImplicitParam(name = "orgId", value = "orgId", required = true, dataType = "String", paramType = "path")
	@GetMapping(value = "api/v1/feign/org/{orgId}")
	public AjaxJson detail(@PathVariable String orgId);

	/**
	 * 验证组织唯一
	 * @param
	 * @return
	 */
	@ApiOperation(value = "组织唯一验证", notes = "组织唯一验证")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "orgName", value = "组织名称", required = false, dataType = "String",paramType = "query"),
		@ApiImplicitParam(name = "orgShortName", value = "组织简称", required = false, dataType = "String",paramType = "query"),
		@ApiImplicitParam(name = "parentId", value = "父组织id", required = true, dataType = "String",paramType = "query"),
	})
	@PostMapping(value = "api/v1/feign/org/validateOrgName")
	public AjaxJson validateOrgName(@RequestBody Org org);

}
