package com.loit.api.service;

import com.loit.common.json.AjaxJson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@FeignClient(name = "loit-portal")
public interface PersonApiService {

    /**
     * 获取人员信息列表
     * @return
     */
    @ApiOperation(value = "获取人员信息列表", notes = "获取人员信息列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNo", value = "当前页码", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "分页大小", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "delFlag", value = "是否删除", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "appId", value = "应用Id", required = false, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "personName", value = "人员名称", required = false, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "userId", value = "用户id", required = false, dataType = "String", paramType = "query")
    })
    @GetMapping(value = "api/v1/feign/person/pageList")
    public AjaxJson list(
            @RequestParam(value="pageNo", required = true) Integer pageNo,
            @RequestParam(value="pageSize", required = true) Integer pageSize,
            @RequestParam(value="delFlag", required = true) String delFlag,
            @RequestParam(value="appId", required = false) Integer appId,
            @RequestParam(value="personName", required = false) String personName,
            @RequestParam(value="userId", required = false) String userId);

    /**
     * 根据组织id获取人员信息列表
     * @return
     */
    @ApiOperation(value = "根据组织id获取人员信息列表", notes = "根据组织id获取人员信息列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNo", value = "当前页码", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "分页大小", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "orgId", value = "组织id", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "delFlag", value = "是否删除", required = true, dataType = "String", paramType = "query")
    })
    @GetMapping(value = "api/v1/feign/person/orgPersonList")
    public AjaxJson getPersonByOrgId(
            @RequestParam(value = "pageNo", required = true) Integer pageNo,
            @RequestParam(value = "pageSize", required = true) Integer pageSize,
            @RequestParam(value = "orgId", required = true) String orgId,
            @RequestParam(value = "delFlag", required = true) String delFlag);
    
    /**
     * @description 根据用户id获取该用户所在组织下的人员列表
     * @return
     */
    @ApiOperation(value = "根据用户id获取该用户所在组织下的人员列表", notes = "根据用户id获取该用户所在组织下的人员列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNo", value = "当前页码", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "分页大小", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "accountId", value = "用户id", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "delFlag", value = "是否删除", required = true, dataType = "String", paramType = "query")
    })
    @GetMapping(value = "api/v1/feign/person/ownOrgPersonList")
    public AjaxJson getListByUserId(
            @RequestParam(value = "pageNo", required = true) Integer pageNo,
            @RequestParam(value = "pageSize", required = true) Integer pageSize,
            @RequestParam(value = "accountId", required = true) String accountId,
            @RequestParam(value = "delFlag", required = true) String delFlag);

    /**
     * 获取人员详细信息
     * @param accountId
     * @return
     */
    @ApiOperation(value = "获取人员详细信息", notes = "根据id来获取人员详细信息")
    @ApiImplicitParam(name = "accountId", value = "id", required = true, dataType = "String", paramType = "path")
    @GetMapping(value = "api/v1/feign/person/{accountId}")
    public AjaxJson detail(@PathVariable(value="accountId") String accountId);

    /**
     * 获取密码策略
     * @param
     * @return
     */
    @ApiOperation(value = "获取密码策略", notes = "获取密码策略")
    @GetMapping(value = "api/v1/feign/person/passwordApolicy")
    public AjaxJson passwordApolicy();

    /**
     * 验证账号唯一
     * @param
     * @return
     */
    @ApiOperation(value = "账号唯一验证", notes = "账号唯一验证")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "账号", required = true, dataType = "String",paramType = "query"),
            @ApiImplicitParam(name = "accountId", value = "账号id", required = false, dataType = "String",paramType = "query")
    })
    @GetMapping(value = "api/v1/feign/person/validateUserId")
    public AjaxJson validateUserId(
            @RequestParam(value = "userId", required = true) String userId,
            @RequestParam(value = "accountId", required = false) Integer accountId);

    /**
     * 验证人员名称唯一
     * @param
     * @return
     */
    @ApiOperation(value = "人员名称唯一验证", notes = "人员名称唯一验证")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "personName", value = "人员名称", required = true, dataType = "String",paramType = "query"),
            @ApiImplicitParam(name = "personBaseId", value = "人员基本信息id", required = false, dataType = "String",paramType = "query")
    })
    @GetMapping(value = "api/v1/feign/person/validatePersonName")
    public AjaxJson validatePersonName(
            @RequestParam(value = "personName", required = true) String personName,
            @RequestParam(value = "personBaseId", required = false) Integer personBaseId);

    /**
     * 密码策略校验
     * @param
     * @return
     */
    @ApiOperation(value = "密码策略校验", notes = "密码策略校验")
    @ApiImplicitParam(name = "passWord", value = "密码", required = true, dataType = "String",paramType = "query")
    @PostMapping(value = "api/v1/feign/person/validatePassword")
    public AjaxJson validatePassword(@RequestParam(value = "passWord", required = true) String passWord);

    /**
     * 获取人员详细信息
     * @param
     * @return
     */
    @ApiOperation(value = "获取当前登录用户", notes = "获取当前登录用户")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "用户登录账号", required = false, dataType = "String", paramType = "query")
    })
    @GetMapping(value = "api/v1/feign/person/getCurretUser")
    public AjaxJson getCurretUser(@RequestParam(value = "userId", required = false) String userId);

    /**
     * 获取所有的账号信息
     * @param
     * @return
     */
    @ApiOperation(value = "获取所有的账号信息", notes = "获取所有的账号信息")
    @GetMapping(value = "api/v1/feign/person/getAccoutInfoList")
    public AjaxJson getAccoutInfoList();

    /**
     * 根据登录账号列表获取账号信息列表
     * @param
     * @return
     */
    @ApiOperation(value = "根据登录账号列表获取账号信息列表", notes = "根据登录账号列表获取账号信息列表")
    @GetMapping(value = "api/v1/feign/person/getAccountByUserIds")
    public AjaxJson getAccountByUserIds(@RequestParam List<String> userIds);

    /**
     * @description 根据角色获取人员分页列表
     * @author xutao
     * @date：2019年10月18日
     * @return
     */
    @GetMapping(value = "api/v1/feign/person/findPersonPageByRole")
    @ApiOperation(value="根据角色获取人员分页列表", notes="根据角色获取人员分页列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNo", value = "当前页码", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "分页大小", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "roleId", value = "角色ID（和角色编码2选1）", required = false, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "roleCode", value = "角色编码（和角色ID2选1）", required = false, dataType = "String", paramType = "query")
    })
    public AjaxJson findPersonPageByRole(
            @RequestParam(value = "pageNo", required = true) Integer pageNo,
            @RequestParam(value = "pageSize", required = true) Integer pageSize,
            @RequestParam(value = "roleId", required = false) Integer roleId,
            @RequestParam(value = "roleCode", required = false) String roleCode);
}
