package com.loit.api.service;

import com.loit.common.json.AjaxJson;
import io.swagger.annotations.ApiOperation;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.Map;

/**
 * @description: websocket消息推送
 * @date: 2020/5/7 10:16
 * @author: wubj
 * @version v1.0
 */
@FeignClient(name = "loit-portal")
public interface WebSocketApiService {

	/**
	 * 广播所有在线用户
	 * @return
	 */
	@PostMapping(value = "api/v1/feign/websocket/broadcast")
	AjaxJson sendMessageBroadcast(@RequestBody String message);

	/**
	 * 发送指定用户
	 * Map<String, String> sendMap（key:userId, value:消息）
	 * @return
	 */
	@PostMapping(value = "api/v1/feign/websocket/toUsers")
	AjaxJson sendMessageToUsers(@RequestBody Map<String, String> sendMap);

	/**
	 * 获取用户连接的websocket服务器信息
	 * @return
	 */
	@GetMapping(value = "api/v1/feign/websocket/websocketServer")
	@ApiOperation(value = "获取用户连接的websocket服务器信息")
	AjaxJson getWebsocketServer();
}
