package com.loit.v1.modules.controller;

import com.loit.common.annotation.BeforeAuditAnnotation;
import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.v1.modules.entity.Message;
import com.loit.v1.modules.service.MessageESService;
import com.loit.v1.modules.service.MessageService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Date;
import java.util.List;


/**
 * 消息管理
 * Created by WBJ on 2020-04-23.
 **/
@RestController
@RequestMapping(value = "/api/v1/message")
@Api(tags = "消息接口",  value = "MessageController")
public class MessageController {

	/**
	 * 日志工具                                                                                                                              
	 */
	private Logger logger = LoggerFactory.getLogger(this.getClass());

	@Autowired
	private MessageService messageService;

	@Autowired
	private MessageESService messageESService;

	/**
	 * 消息详情
	 */
	@ApiOperation(value = "消息详情", notes = "根据id获取消息详情信息")
	@ApiImplicitParam(paramType = "path", name = "id", value = "消息id", required = true, dataType = "Long")
	@RequestMapping(value = "/{id}", method = RequestMethod.GET)
	public AjaxJson get(@PathVariable("id") Integer id) {
		Message notice = messageService.get(id);
		AjaxJson json = new AjaxJson();
		json.put("obj", notice);
		return json;
	}

	/**
	 * 消息分页列表
	 */
	@ApiOperation(value = "消息分页列表", notes = "消息分页列表")
	@ApiImplicitParams({
			@ApiImplicitParam(paramType = "query", name = "pageNo", value = "当前页码", required = true, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "pageSize", value = "分页大小", required = true, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "noticeTopic", value = "消息标题", required = false, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "noticeType", value = "消息类型", required = false, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "noticeSource", value = "消息来源", required = false, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "noticeLevel", value = "紧急程度（1一般，2紧急）", required = false, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "noticeStatus", value = "状态（0未读，1已读）", required = false, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "showFlag", value = "是否展示所有接收人信息(0 否 1是)", required = false, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "createTimeStart", value = "开始时间", required = false, dataType = "Date"),
			@ApiImplicitParam(paramType = "query", name = "createTimeEnd", value = "结束时间", required = false, dataType = "Date"),
			@ApiImplicitParam(paramType = "query", name = "orderByName", value = "排序字段名称", required = false, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "orderBy", value = "排序（asc/desc）", required = false, dataType = "String")
	})
	@RequestMapping(value = "/page", method = RequestMethod.GET)
	public AjaxJson findPage(@ApiIgnore Message notice, HttpServletRequest request, HttpServletResponse response) {
		AjaxJson json = new AjaxJson();
		Page<Message> page = messageService.findPage(new Page<Message>(request, response), notice);
		json.put("page", page);
		return json;
	}

	/**
	 * 消息推送
	 */
	@ApiOperation(value = "消息推送", notes = "消息推送")
	@PostMapping("/propel")
	@BeforeAuditAnnotation
	public AjaxJson propel(@RequestBody Message notice) {
		AjaxJson json = new AjaxJson();
		try {
			messageService.propel(notice);
			json.setMsg("推送成功");
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("推送异常");
		}
		return json;
	}

	/**
	 * 统计未读消息数量
	 */
	@ApiOperation(value = "统计未读消息数量", notes = "统计未读消息数量")
	@GetMapping("/unread")
	public AjaxJson getUnreadList() {
		AjaxJson json = new AjaxJson();
		List<Message> list = messageService.getUnreadList();
		json.put("list", list);
		json.put("count", list.size());
		return json;
	}

	/**
	 * 标记消息为已读
	 */
	@ApiOperation(value = "标记消息为已读", notes = "标记消息为已读")
	@PostMapping("/updateStatus")
	public AjaxJson updateStatus(@RequestBody Message notice) {
		AjaxJson json = new AjaxJson();
		try {
			messageService.updateStatus(notice);
			json.setMsg("标记成功");
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("标记异常");
		}
		return json;
	}

	@ApiOperation(value = "历史消息分页列表", notes = "历史消息分页列表")
	@ApiImplicitParams({
			@ApiImplicitParam(paramType = "query", name = "pageNo", value = "当前页码", required = true, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "pageSize", value = "分页大小", required = true, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "noticeTopic", value = "消息标题", required = false, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "noticeType", value = "消息类型", required = false, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "noticeLevel", value = "紧急程度（1一般，2紧急）", required = false, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "createTimeStart", value = "开始时间", required = false, dataType = "Date"),
			@ApiImplicitParam(paramType = "query", name = "createTimeEnd", value = "结束时间", required = false, dataType = "Date")
	})
	@RequestMapping(value = "/historyPage", method = RequestMethod.GET)
	public AjaxJson findHistoryPage(@RequestParam(value = "pageNo", required = true) Integer pageNo,
									@RequestParam(value = "pageSize", required = true) Integer pageSize,
									@RequestParam(value = "noticeTopic", required = false)String noticeTopic,
									@RequestParam(value = "noticeType", required = false)String noticeType,
									@RequestParam(value = "noticeLevel", required = false)String noticeLevel,
									@RequestParam(value = "createTimeStart", required = false)Date createTimeStart,
									@RequestParam(value = "createTimeEnd", required = false)Date createTimeEnd,
									HttpServletRequest request, HttpServletResponse response
									) {
		AjaxJson json = new AjaxJson();
		try {
			json = messageESService.findHistoryPage(pageNo, pageSize, noticeTopic,
					noticeType, noticeLevel,createTimeStart, createTimeEnd);
		} catch (Exception e) {
			Message notice = new Message();
			notice.setNoticeTopic(noticeTopic);
			notice.setNoticeType(noticeType);
			notice.setNoticeLevel(noticeLevel);
			notice.setCreateTimeStart(createTimeStart);
			notice.setCreateTimeEnd(createTimeEnd);
			Page<Message> page = messageService.findPage(new Page<Message>(request, response), notice);
			json.put("page", page);
			logger.error("es查询消息失败：{}", e.getMessage());
		}
		return json;
	}
}
