package com.loit.v1.modules.controller;

import com.loit.common.json.AjaxJson;                                                                                                        
import com.loit.common.persistence.Page;
import com.loit.v1.modules.entity.MessageReceive;
import com.loit.v1.modules.service.MessageReceiveService;
import io.swagger.annotations.*;
import org.slf4j.Logger;                                                                                                                     
import org.slf4j.LoggerFactory;                                                                                                              
import org.springframework.beans.factory.annotation.Autowired;                                                                               
import org.springframework.web.bind.annotation.*;                                                                                            
import springfox.documentation.annotations.ApiIgnore;                                                                                        
import javax.servlet.http.HttpServletRequest;                                                                                                
import javax.servlet.http.HttpServletResponse;                                                                                               
import java.util.List;                                                                                                                       
                                                                                                                                             
                                                                                                                                             


/**
 * 消息接收接口
 * Created by WBJ on 2020-04-23.
 **/
@RestController                                                                                                                              
@RequestMapping(value = "/api/v1/messageReceive")
@Api(tags = "消息接收接口",  value = "MessageReceiveController")
public class MessageReceiveController {
                                                                                                                                             
	/**                                                                                                                                      
	 * 日志工具                                                                                                                              
	 */                                                                                                                                      
	private Logger logger = LoggerFactory.getLogger(this.getClass());                                                                        
	                                                                                                                                         
	@Autowired                                                                                                                               
	private MessageReceiveService messageReceiveService;
	                                                                                                                                         
	/**                                                                                                                                      
	 * 消息接收列表                                                                                                                              
	 */                                                                                                                                      
	@ApiOperation(value = "消息接收列表", notes = "消息接收列表")                                                                                
	@ApiImplicitParams({                                                                                                                     
	        @ApiImplicitParam(paramType = "query", name = "noticeId", value = "消息id", required = false, dataType = "Integer"),
	        @ApiImplicitParam(paramType = "query", name = "receiveId", value = "接收人", required = false, dataType = "String"),
	        @ApiImplicitParam(paramType = "query", name = "noticeStatus", value = "状态（0未读，1已读）", required = false, dataType = "String"),
	        @ApiImplicitParam(paramType = "query", name = "createTime", value = "创建时间", required = false, dataType = "Date"),
	        @ApiImplicitParam(paramType = "query", name = "handleStatus", value = "处理状态（0未处理，1已处理）", required = false, dataType = "String")
	})
	@RequestMapping(value = "/list", method = RequestMethod.GET)                                                                           
	public AjaxJson findAll(@ApiIgnore MessageReceive noticeReceive) {
		AjaxJson json = new AjaxJson();                                                                                                      
		List<MessageReceive> list = messageReceiveService.findList(noticeReceive);
		json.put("list", list);                                                                                                            
		return json;                                                                                                                         
	}                                                                                                                                        
                                                                                                                                             
	/**                                                                                                                                      
	 * 消息接收分页列表                                                                                                                          
	 */                                                                                                                                      
	@ApiOperation(value = "消息接收分页列表", notes = "消息接收分页列表")                                                                        
	@ApiImplicitParams({                                                                                                                     
	        @ApiImplicitParam(paramType = "query", name = "pageNo", value = "当前页码", required = true, dataType = "String"),  
	        @ApiImplicitParam(paramType = "query", name = "pageSize", value = "分页大小", required = true, dataType = "String"),
	        @ApiImplicitParam(paramType = "query", name = "noticeId", value = "消息id", required = false, dataType = "Integer"),
	        @ApiImplicitParam(paramType = "query", name = "receiveId", value = "接收人", required = false, dataType = "String"),
	        @ApiImplicitParam(paramType = "query", name = "noticeStatus", value = "状态（0未读，1已读）", required = false, dataType = "String"),
	        @ApiImplicitParam(paramType = "query", name = "createTime", value = "创建时间", required = false, dataType = "Date"),
	        @ApiImplicitParam(paramType = "query", name = "handleStatus", value = "处理状态（0未处理，1已处理）", required = false, dataType = "String")
	})
	@RequestMapping(value = "/page", method = RequestMethod.GET)                                                                           
	public AjaxJson findPage(@ApiIgnore MessageReceive noticeReceive, HttpServletRequest request, HttpServletResponse response) {
		AjaxJson json = new AjaxJson();                                                                                                      
		Page<MessageReceive> page = messageReceiveService.findPage(new Page<MessageReceive>(request, response), noticeReceive);
		json.put("page", page);                                                                                                            
		return json;                                                                                                                         
	}                                                                                                                                        
                                                                                                                                             
	/**                                                                                                                                      
	 * 消息接收详情                                                                                                                              
	 */                                                                                                                                    
	@ApiOperation(value = "消息接收详情", notes = "根据id获取消息接收详情信息")                                                                  
	@ApiImplicitParam(paramType = "path", name = "id", value = "消息接收id", required = true, dataType = "Long")                         
	@RequestMapping(value = "/{id}", method = RequestMethod.GET)                                                                           
	public AjaxJson get(@PathVariable("id") Integer id) {                                                                              
		MessageReceive noticeReceive = messageReceiveService.get(id);
		AjaxJson json = new AjaxJson();                                                                                                      
		json.put("obj", noticeReceive);                                                                                                         
		return json;                                                                                                                         
	}                                                                                                                                        
                                                                                                                                             
	/**                                                                                                                                      
	 * 消息接收保存                                                                                                                              
	 */                                                                                                                                      
	@ApiOperation(value = "消息接收保存", notes = "消息接收保存")                                                                                
	@RequestMapping(value = "/save", method = RequestMethod.POST)                                                                          
	public AjaxJson save(@RequestBody MessageReceive noticeReceive) {
		AjaxJson json = new AjaxJson();                                                                                                      
		try {                                                                                                                                
			messageReceiveService.insert(noticeReceive);
			json.setMsg("保存成功");                                                                                                       
		} catch (Exception e) {                                                                                                              
			e.printStackTrace();                                                                                                             
			return AjaxJson.returnExceptionInfo("保存异常");                                                                               
		}                                                                                                                                    
		return json;                                                                                                                         
	}                                                                                                                                        
	                                                                                                                                         
	/**                                                                                                                                      
	 * 消息接收修改                                                                                                                              
	 */                                                                                                                                      
	@ApiOperation(value = "消息接收修改", notes = "消息接收修改")                                                                                
	@RequestMapping(value = "/update", method = RequestMethod.POST)                                                                        
	public AjaxJson update(@RequestBody MessageReceive noticeReceive) {
		AjaxJson json = new AjaxJson();                                                                                                      
		try {                                                                                                                                
			messageReceiveService.update(noticeReceive);
			json.setMsg("修改成功");                                                                                                       
		} catch (Exception e) {                                                                                                              
			e.printStackTrace();                                                                                                             
			return AjaxJson.returnExceptionInfo("修改异常");                                                                               
		}                                                                                                                                    
		return json;                                                                                                                         
	}                                                                                                                                        
	                                                                                                                                         
	/**                                                                                                                                      
	 * 消息接收删除                                                                                                                              
	 */                                                                                                                                      
	@ApiOperation(value = "消息接收删除", notes = "消息接收删除")                                                                                
	@ApiImplicitParam(paramType = "path", name = "id", value = "消息接收id", required = true, dataType = "Long")                         
	@RequestMapping(value = "/{id}", method = RequestMethod.DELETE)                                                                        
	public AjaxJson delete(@PathVariable("id") String ids) {                                                                               
		AjaxJson json = new AjaxJson();                                                                                                      
		try {                                                                                                                                
			messageReceiveService.deleteAll(ids, new MessageReceive());
			json.setMsg("删除成功!");                                                                                                      
		} catch (Exception e) {                                                                                                              
			e.printStackTrace();                                                                                                             
			return AjaxJson.returnExceptionInfo("删除失败" + e.getMessage());                                                              
		}                                                                                                                                    
		return json;                                                                                                                         
	}                                                                                                                                        
	                                                                                                                                         
                                                                                                                                             
}                                                                                                                                            
