package com.loit.v1.modules.entity;

import com.fasterxml.jackson.annotation.JsonFormat;
import org.springframework.data.annotation.Id;
import org.springframework.data.elasticsearch.annotations.DateFormat;
import org.springframework.data.elasticsearch.annotations.Document;
import org.springframework.data.elasticsearch.annotations.Field;
import org.springframework.data.elasticsearch.annotations.FieldType;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * @description: 历史消息数据表
 * @date: 2020/5/28 13:33
 * @author: wubj
 * @version v1.0
 */
@Document(indexName = "notice", type = "message")
public class MessageES implements Serializable {

	/**
	 * 消息id
	 */
	@Id
	private Integer id;

	/**
	 * 消息类型
	 */
	@Field
	private String noticeType;

	/**
	 * 消息类型名称
	 */
	@Field
	private String noticeTypeName;

	/**
	 * 消息标题
	 */
	@Field(type = FieldType.Text, analyzer = "ik_max_word", searchAnalyzer = "ik_max_word")
	private String noticeTopic;

	/**
	 * 消息内容
	 */
	@Field
	private String noticeContent;

	/**
	 * 消息来源
	 */
	@Field
	private String noticeSource;

	/**
	 * 消息来源名称
	 */
	@Field
	private String noticeSourceName;

	/**
	 * 发送人
	 */
	@Field
	private String sendId;

	/**
	 * 发送人名称
	 */
	@Field
	private String sendName;

	/**
	 * 紧急程度（1一般，2紧急）
	 */
	@Field
	private String noticeLevel;

	/**
	 * 广播消息标记
	 */
	@Field
	private String allFlag;

	/**
	 * 状态（未读0，已读1）
	 */
	@Field
	private String noticeStatus;

	@Field(type = FieldType.Date, format = DateFormat.date_optional_time)
	@JsonFormat(shape = JsonFormat.Shape.STRING, pattern ="yyyy-MM-dd'T'HH:mm:ss",timezone="GMT+8")
	private Date createTime;

	/**
	 * 接收人
	 */
	@Field(type =FieldType.Nested)
	private List<MessageReveiveES> receive;

	public void setId(Integer id){
		this.id = id;
	}

	public Integer getId(){
		return this.id;
	}

	public void setNoticeType(String noticeType){
		this.noticeType = noticeType;
	}

	public String getNoticeType(){
		return this.noticeType;
	}

	public String getNoticeTopic() {
		return noticeTopic;
	}

	public void setNoticeTopic(String noticeTopic) {
		this.noticeTopic = noticeTopic;
	}

	public String getNoticeContent() {
		return noticeContent;
	}

	public void setNoticeContent(String noticeContent) {
		this.noticeContent = noticeContent;
	}

	public String getNoticeSource() {
		return noticeSource;
	}

	public void setNoticeSource(String noticeSource) {
		this.noticeSource = noticeSource;
	}

	public String getSendId() {
		return sendId;
	}

	public void setSendId(String sendId) {
		this.sendId = sendId;
	}

	public String getSendName() {
		return sendName;
	}

	public void setSendName(String sendName) {
		this.sendName = sendName;
	}

	public String getNoticeLevel() {
		return noticeLevel;
	}

	public void setNoticeLevel(String noticeLevel) {
		this.noticeLevel = noticeLevel;
	}

	public String getAllFlag() {
		return allFlag;
	}

	public void setAllFlag(String allFlag) {
		this.allFlag = allFlag;
	}

	public String getNoticeStatus() {
		return noticeStatus;
	}

	public void setNoticeStatus(String noticeStatus) {
		this.noticeStatus = noticeStatus;
	}

	public String getNoticeTypeName() {
		return noticeTypeName;
	}

	public void setNoticeTypeName(String noticeTypeName) {
		this.noticeTypeName = noticeTypeName;
	}

	public String getNoticeSourceName() {
		return noticeSourceName;
	}

	public void setNoticeSourceName(String noticeSourceName) {
		this.noticeSourceName = noticeSourceName;
	}

	public Date getCreateTime() {
		return createTime;
	}

	public void setCreateTime(Date createTime) {
		this.createTime = createTime;
	}

	public List<MessageReveiveES> getReceive() {
		return receive;
	}

	public void setReceive(List<MessageReveiveES> receive) {
		this.receive = receive;
	}
}
