package com.loit.v1.modules.schedule;

import com.loit.common.utils.ListUtil;
import com.loit.v1.modules.constant.NoticeConstants;
import com.loit.v1.modules.service.MessageReceiveService;
import com.loit.v1.modules.service.MessageService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * @description: 消息定时器
 * @date: 2020/4/26 10:38
 * @author: wubj
 * @version v1.0
 */
@Component
public class MessageSchedule {

    private Logger log = LoggerFactory.getLogger(this.getClass());

    @Autowired
    private MessageService messageService;

    @Autowired
    private MessageReceiveService messageReceiveService;

    private static ExecutorService singleThreadExecutor = Executors
            .newSingleThreadExecutor();

    /**
     * 定时任务：删除三个月以前的已读消息和广播消息
     */
    // @Scheduled(cron = "0 0 1 * * ?")
    public void scheduledTask() {
        synchronized (NoticeConstants.NOTICE_TASK) {
            singleThreadExecutor.execute(new Runnable() {
                @Override
                public void run() {
                    doNoticeTask();
                }
            });
        }
    }

    public void doNoticeTask() {
        log.info("删除三个月以前的已读消息和广播消息开始执行");
        long begin = System.currentTimeMillis();

        // 删除消息接收表
        List<Long> ids = messageReceiveService.getRecentNoticeIds();
        if (ListUtil.isNotEmpty(ids)) {
            messageReceiveService.deleteByIds(ids);
        }

        // 删除消息表
        List<Long> noticeIds = messageService.getRecentNoticeIds();
        if (ListUtil.isNotEmpty(noticeIds)) {
            messageService.deleteByIds(noticeIds);
        }

        long end = System.currentTimeMillis();
        log.info("删除三个月以前的已读消息和广播消息执行结束，花费时间为" + (end - begin) + "毫秒");
    }
}
