package com.loit.v1.modules.service;

import com.alibaba.fastjson.JSON;
import com.loit.api.service.PersonApiService;
import com.loit.common.json.AjaxJson;
import com.loit.common.utils.ListUtil;
import com.loit.common.utils.StringUtils;
import com.loit.modules.sys.entity.User;
import com.loit.v1.modules.dao.MessageESDao;
import com.loit.v1.modules.entity.MessageES;
import com.loit.v1.modules.entity.MessageReveiveES;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.query.RangeQueryBuilder;
import org.elasticsearch.index.query.WildcardQueryBuilder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @description: 历史消息services
 * @date: 2020/6/17 19:07
 * @author: wubj
 * @version v1.0
 */
@Service
public class MessageESService {

    @Autowired
    private MessageESDao messageESDao;

    @Autowired
    private PersonApiService personApiService;

    /**
     * 分页查询
     * @return
     */
    public AjaxJson findHistoryPage(Integer pageNo, Integer pageSize, String noticeTopic, String noticeType,
                                    String  noticeLevel, Date createTimeStart, Date createTimeEnd) {

        Sort sort = Sort.by(Sort.Direction.DESC,"createTime");
//		Sort.Order statusOrder = new Sort.Order(Sort.Direction.ASC,"receive.noticeStatus");
//		Sort.Order timeOrder = new Sort.Order(Sort.Direction.DESC,"createTime");
//		Sort sort = Sort.by(statusOrder, timeOrder);

        Pageable pageable = PageRequest.of(pageNo-1, pageSize, sort);

        // 查询条件
        BoolQueryBuilder bql = QueryBuilders.boolQuery();
        User user = this.getCurretUserId();
        if (user != null) {
            bql.must(QueryBuilders.matchPhraseQuery("receive.receiveId", user.getUserId()));
        }
        if (StringUtils.isNotEmpty(noticeTopic)) {
            bql.must(QueryBuilders.matchQuery( "noticeTopic", noticeTopic));
        }
        if (StringUtils.isNotEmpty(noticeType)) {
            bql.must(QueryBuilders.matchPhraseQuery("noticeType", noticeType));
        }
        if (StringUtils.isNotEmpty(noticeLevel)) {
            bql.must(QueryBuilders.matchPhraseQuery("noticeLevel", noticeLevel));
        }
        if (createTimeStart != null || createTimeEnd != null) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
            RangeQueryBuilder rqb = QueryBuilders.rangeQuery("createTime");
            if (createTimeStart != null) {
                String start = sdf.format(createTimeStart);
                rqb.from(start);
            }
            if (createTimeEnd != null) {
                String end = sdf.format(createTimeEnd);
                rqb.to(end);
            }
            bql.must(rqb);
        }

        // 分页查询
        Page<MessageES> page = messageESDao.search(bql, pageable);

        // 组装数据
        AjaxJson json = new AjaxJson();
        Map<String, Object> map = new HashMap<>();
        if (page != null) {
            if(ListUtil.isNotEmpty(page.getContent())) {
                List<MessageES> list = page.getContent();
                for (MessageES messageES : list) {
                    if (ListUtil.isNotEmpty(messageES.getReceive())) {
                        for (MessageReveiveES messageReveiveES : messageES.getReceive()) {
                            if (user.getUserId().equals(messageReveiveES.getReceiveId())) {
                                messageES.setNoticeStatus(messageReveiveES.getNoticeStatus());
                                break;
                            }
                        }
                    }
                }
            }
            map.put("pageNo", page.getPageable().getPageNumber() + 1);
            map.put("pageSize", page.getPageable().getPageSize());
            map.put("count", page.getTotalElements());
            map.put("pageTotal", page.getTotalPages());
            map.put("list", page.getContent());
        }
        json.put("page", map);

        return json;
    }

    /**
     * 获取当前用户信息
     * @return
     */
    private User getCurretUserId() {
        //获取当前用户
        User user = null;
        AjaxJson personJson = personApiService.getCurretUser(null);
        if (personJson.isSuccess()) {
            Object obj = personJson.getBody();
            if (obj != null) {
                LinkedHashMap accout = (LinkedHashMap) ((LinkedHashMap) obj).get("accout");
                user = JSON.parseObject(JSON.toJSONString(accout), User.class);
            }
        }
        return user;
    }

}
