package com.loit.v1.modules.service;

import com.loit.v1.modules.dao.MessageReceiveDao;
import com.loit.v1.modules.entity.MessageReceive;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.loit.common.service.CrudService;

import java.util.List;


/**
 * 消息接收表服务实现.
 * Created by WBJ on 2020-04-23.
 **/
@Service
@Transactional(readOnly = true)
public class MessageReceiveService extends CrudService<MessageReceiveDao, MessageReceive> {

    /**
     * 标记消息为已读
     * @param noticeIds
     */
    @Transactional(readOnly = false)
    public void updateStatus(String receiveId, List<Long> noticeIds) {
        dao.updateStatus(receiveId, noticeIds);
    }

    /**
     * 获取未读消息的数量
     * @param receiveId
     * @return
     */
    public int getUnreadCount(String receiveId) {
        return dao.getUnreadCount(receiveId);
    }

    /**
     * 根据ids删除数据
     */
    @Transactional(readOnly = false)
    public void deleteByIds(List<Long> noticeIds) {
        dao.deleteByIds(noticeIds);
    }

    /**
     * 根据noticeIds删除数据
     */
    @Transactional(readOnly = false)
    public void deleteByNoticeIds(List<Long> noticeIds) {
        dao.deleteByNoticeIds(noticeIds);
    }

    /**
     * 获取近期消息ids
     * @return
     */
    public List<Long> getRecentNoticeIds() {
        return dao.getRecentNoticeIds();
    }

    public void insertBatch(List<MessageReceive> receiveList) {
        dao.insertBatch(receiveList);
    }
}