package com.loit.v1.modules.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.loit.api.dto.AccoutInfo;
import com.loit.api.dto.Colour;
import com.loit.api.dto.DictData;
import com.loit.api.dto.PersonBase;
import com.loit.api.service.*;
import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.service.CrudService;
import com.loit.common.utils.JSONUtils;
import com.loit.common.utils.ListUtil;
import com.loit.common.utils.StringUtils;
import com.loit.modules.sys.entity.User;
import com.loit.v1.modules.config.WebSocketClient;
import com.loit.v1.modules.constant.NoticeConstants;
import com.loit.v1.modules.dao.MessageDao;
import com.loit.v1.modules.dao.MessageESDao;
import com.loit.v1.modules.entity.Message;
import com.loit.v1.modules.entity.MessageES;
import com.loit.v1.modules.entity.MessageReceive;
import com.loit.v1.modules.entity.MessageReveiveES;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;


/**
 * 消息表服务实现.
 * Created by WBJ on 2020-04-23.
 **/
@Service
public class MessageService extends CrudService<MessageDao, Message> {

    @Autowired
    private MessageReceiveService messageReceiveService;

    @Autowired
    private DictDataApiService dictDataApiService;

    @Autowired
    private ColourApiService colourApiService;

    @Autowired
    private PersonApiService personApiService;

    @Autowired
    private WebSocketApiService webSocketApiService;

    @Autowired
    private JPushApiService jPushApiService;

    @Autowired
    private MessageESDao messageESDao;

    // 批量存储数据个数
    private static final int NOTICE_NUM = 5000;

    // 字典map
    private Map<String, String> dictMap = new HashMap<>();

    /**
     * 消息中心分页列表
     * @param page 分页对象
     * @param notice
     * @return
     */
    public Page<Message> findPage(Page<Message> page, Message notice) {
        notice.setPage(page);
        //查询所有接收人
        if (notice.getShowFlag() != null && "1".equals(notice.getShowFlag())){
            notice.setReceiveId(null);
        }else if (StringUtils.isEmpty(notice.getReceiveId())) {
            // 当前登录用户名
            User user = this.getCurretUserId();
            notice.setReceiveId(user.getUserId());
        }
        List<Message> list = dao.findList(notice);
        this.assemblyNoticeData(list);
        page.setList(list);
        return page;
    }

    /**
     * 消息详情
     * @param id
     * @return
     */
    public Message get(Integer id) {
        Message notice = dao.get(id);
        if (notice != null) {
            this.getDictMap();
            notice.setChildren(new ArrayList());
            notice.setNoticeTypeName(dictMap.get(notice.getNoticeType()));
            notice.setNoticeSourceName(dictMap.get(notice.getNoticeSource()));
            this.assemblyNoticeLevelColour(notice);
        }
        return notice;
    }

    /**
     * 组装列表数据
     * @param list
     * @return
     */
    private List<Message> assemblyNoticeData(List<Message> list) {
        if (ListUtil.isNotEmpty(list)) {
            this.getDictMap();
            List<String> receiveIds = list.stream().map(p->p.getReceiveId()).collect(Collectors.toList());
            List<PersonBase> personBases = this.getAccountByUserIds(receiveIds);
            Map<String, String> personMap = personBases.stream().collect(Collectors.toMap(PersonBase::getUserId, PersonBase::getPersonName));

            List children = new ArrayList();
            for (Message entity : list) {
                if (StringUtils.isNotEmpty(entity.getNoticeType()) && StringUtils.isNotEmpty(dictMap.get(entity.getNoticeType()))) {
                    entity.setNoticeTypeName(dictMap.get(entity.getNoticeType()));
                }
                if (StringUtils.isNotEmpty(entity.getNoticeSource()) && StringUtils.isNotEmpty(dictMap.get(entity.getNoticeSource()))) {
                    entity.setNoticeSourceName(dictMap.get(entity.getNoticeSource()));
                }
                if (entity.getReceiveId() != null && personMap != null && personMap.get(entity.getReceiveId()) != null) {
                    entity.setReceiveName(personMap.get(entity.getReceiveId()));
                }
                this.assemblyNoticeLevelColour(entity);
                entity.setChildren(children);
            }
        }
        return list;
    }

    private void assemblyNoticeLevelColour(Message message){
        if(message!=null && StringUtils.isNotBlank(message.getNoticeLevel())){
            AjaxJson json =colourApiService.findByDictCodeAndDataCode(NoticeConstants.NOTICE_GRADE,message.getNoticeLevel());
            if (json.isSuccess()) {
                Object obj = json.getBody();
                if (obj != null) {
                    LinkedHashMap map = (LinkedHashMap)((LinkedHashMap) obj).get("colour");
                    Colour colour = JSON.parseObject(JSON.toJSONString(map), Colour.class);
                    if(colour!=null && StringUtils.isNotBlank(colour.getColourValue())){
                        message.setNoticeLevelColour(colour.getColourValue());
                    }
                }
            }

        }
    }

    /**
     * 获取字典数据
     */
    private void getDictMap() {
        if (dictMap == null || dictMap.size() == 0) {
            // 获取字典信息
            List<DictData> typeDict = this.getDictDataList(NoticeConstants.NOTICE_TYPE);
            List<DictData> sourceDict = this.getDictDataList(NoticeConstants.NOTICE_SOURCE);
            for (DictData dictData : typeDict) {
                dictMap.put(dictData.getDataCode(), dictData.getDataName());
            }
            for (DictData dictData : sourceDict) {
                dictMap.put(dictData.getDataCode(), dictData.getDataName());
            }
        }
    }

    /**
     * 根据dictCode获取字典值
     * @param dictCode
     * @return
     */
    private List<DictData> getDictDataList(String dictCode) {
        List<DictData> dictDataList = new ArrayList<>();
        AjaxJson json = dictDataApiService.listDictDataByDictId(dictCode);
        if (json.isSuccess()) {
            Object obj = json.getBody();
            if (obj != null) {
                List<LinkedHashMap> map = (List<LinkedHashMap>)((LinkedHashMap) obj).get("dictDataList");
                dictDataList = JSON.parseArray(JSON.toJSONString(map), DictData.class);
            }
        }
        return dictDataList;
    }

    /**
     * 获取当前用户信息
     * @return
     */
    private User getCurretUserId() {
        User user = new User();
        //获取当前用户
        AjaxJson personJson = personApiService.getCurretUser(null);
        if (personJson.isSuccess()) {
            Object obj = personJson.getBody();
            if (obj != null) {
                LinkedHashMap accout = (LinkedHashMap) ((LinkedHashMap) obj).get("accout");
                user = JSON.parseObject(JSON.toJSONString(accout), User.class);
            }
        }
        return user;
    }

    /**
     * 获取所有的账号信息
     * @return
     */
    private List<PersonBase> getAccountByUserIds(List<String> receiveIds) {
        List<PersonBase> accoutInfos = new ArrayList<>();
        AjaxJson json = personApiService.getAccountByUserIds(receiveIds);
        if (json.isSuccess()) {
            Object obj = json.getBody();
            if (obj != null) {
                List<LinkedHashMap> map = (List<LinkedHashMap>)((LinkedHashMap) obj).get("list");
                accoutInfos = JSON.parseArray(JSON.toJSONString(map), PersonBase.class);
            }
        }
        return accoutInfos;
    }

    /**
     * 获取所有的账号信息
     * @return
     */
    private List<AccoutInfo> getAccoutInfoList() {
        List<AccoutInfo> accoutInfos = new ArrayList<>();
        AjaxJson json = personApiService.getAccoutInfoList();
        if (json.isSuccess()) {
            Object obj = json.getBody();
            if (obj != null) {
                List<LinkedHashMap> map = (List<LinkedHashMap>)((LinkedHashMap) obj).get("list");
                accoutInfos = JSON.parseArray(JSON.toJSONString(map), AccoutInfo.class);
            }
        }
        return accoutInfos;
    }

    /**
     * 推送消息
     * @param notice
     */
    @Transactional(readOnly = false)
    public void propel(Message notice) {
        // 保存消息实体
        this.save(notice);

        try {
            this.publish(notice); // 推送消息
        } catch (Exception e) {
            logger.error("消息推送异常：{}", e.getMessage());
        }
    }

    /**
     * 批量插入数据到消息接收表
     * @param receiveList
     */
    private void insertNoticeReceiveList(List<MessageReceive> receiveList){
        int listSize = receiveList.size();
        int toIndex = NOTICE_NUM;
        for (int i = 0; i < listSize; i += NOTICE_NUM) {
            // 作用为toIndex最后没有1000条数据则剩余几条newList中就装几条
            if (i + NOTICE_NUM > listSize) {
                toIndex = listSize - i;
            }
            List<MessageReceive> newList = receiveList.subList(i, i + toIndex);
            // 批量插入
            messageReceiveService.insertBatch(newList);
        }
    }

    /**
     * 把数据转存到历史表
     */
    private void saveHistoryData(Message message) {
        MessageES messageES = new MessageES();
        messageES.setId(message.getId());
        messageES.setNoticeType(message.getNoticeType());
        messageES.setNoticeTypeName(message.getNoticeTypeName());
        messageES.setNoticeTopic(message.getNoticeTopic());
        messageES.setNoticeContent(message.getNoticeContent());
        messageES.setSendId(message.getSendId());
        messageES.setSendName(message.getSendName());
        messageES.setNoticeSource(message.getNoticeSource());
        messageES.setNoticeSourceName(message.getNoticeSourceName());
        messageES.setNoticeLevel(message.getNoticeLevel());
        messageES.setCreateTime(message.getCreateTime());
        if (message.getReceiveIds() != null) {
            String[] receiveIds = message.getReceiveIds().split(",");
            List<MessageReveiveES> list = new ArrayList<>();
            for (String receiveId : receiveIds) {
                MessageReveiveES messageReveiveES = new MessageReveiveES();
                messageReveiveES.setReceiveId(receiveId);
                list.add(messageReveiveES);
            }
            messageES.setReceive(list);
        }
        try {
            messageESDao.save(messageES);
        } catch (Exception e) {
            logger.error("消息存到ES异常：{}", e.getMessage());
        }
    }

    /**
     * 统计未读消息数量
     * @return
     */
    public List<Message> getUnreadList() {
        Message notice = new Message();
        // 当前登录用户名
        User user = this.getCurretUserId();
        notice.setReceiveId(user.getUserId());
        notice.setNoticeStatus("0");
        List<Message> list = dao.findList(notice);
        this.assemblyNoticeData(list);
        return list;
    }

    /**
     * 标记消息为已读
     * @param noticeDto
     */
    @Transactional(readOnly = false)
    public void updateStatus(Message noticeDto) {
        List<Long> ids = new ArrayList<>();
        if (noticeDto.getNoticeTypeId() != null) {
            Long id = dao.findIdByNoticeTypeId(noticeDto.getNoticeTypeId());
            ids.add(id);
        } else {
            String[] strIds = noticeDto.getNoticeIds().split(",");
            for (String strId : strIds) {
                ids.add(Long.valueOf(strId));
            }
        }

        String userId = noticeDto.getReceiveId();
        if (StringUtils.isEmpty(userId)) {
            User user = this.getCurretUserId();
            userId = user.getUserId();
        }

        if (ids != null && ids.size() > 0 && userId != null) {
            // 标记消息为已读
            messageReceiveService.updateStatus(userId, ids);
        }
    }

    /**
     * 根据ids删除数据
     */
    @Transactional(readOnly = false)
    public void deleteByIds(List<Long> noticeIds) {
        dao.deleteByIds(noticeIds);
    }

    /**
     * 获取近期消息ids
     * @return
     */
    public List<Long> getRecentNoticeIds() {
        return dao.getRecentNoticeIds();
    }

    /**
     * 新增或修改消息
     * @param notice
     */
    @Transactional(readOnly = false)
    public void save(Message notice) {
        if (notice.getId() != null) {
            // 修改消息实体
            notice.setUpdateTime(new Date());
            dao.update(notice);
            // 修改接收人信息
            messageReceiveService.deleteByNoticeIds(Arrays.asList(notice.getId().longValue()));
        } else {
            if (!StringUtils.isNotEmpty(notice.getSendId())) {
                // 保存消息实体
                User user = this.getCurretUserId();
                notice.setSendId(user.getUserId());
                notice.setSendName(user.getPersonName());
            }
            notice.setUpdateTime(new Date());
            notice.setCreateTime(new Date());
            dao.insert(notice);
        }

        // 判断是否保存消息接收表
        int noticeId = notice.getId();
        String allFlag = notice.getAllFlag();
        List<String> accoutList = new ArrayList<>();
        List<MessageReceive> receiveList = new ArrayList<>();

        // 是广播消息
        if ("1".equals(allFlag)) {
            // 获取所有的账号信息
            List<AccoutInfo> accoutInfos = this.getAccoutInfoList();
            if (ListUtil.isNotEmpty(accoutInfos)) {
                accoutList = accoutInfos.stream().map(p->p.getUserId()).collect(Collectors.toList());
            }
            // 不是广播消息
        } else if ("0".equals(allFlag) && StringUtils.isNotEmpty(notice.getReceiveIds())) {
            // 存储消息接收人信息
            String[] receiveIds = notice.getReceiveIds().split(",");
            accoutList = Arrays.asList(receiveIds);
        }

        if (ListUtil.isNotEmpty(accoutList)) {
            notice.setReceiveIds(StringUtils.join(accoutList, ","));
            for (String receiveId : accoutList) {
                MessageReceive noticeReceive = new MessageReceive();
                noticeReceive.setNoticeId(noticeId);
                noticeReceive.setReceiveId(receiveId);
                noticeReceive.setNoticeStatus("0");
                noticeReceive.setCreateTime(new Date());
                receiveList.add(noticeReceive);
            }
            // 批量存储消息接收表信息
            this.insertNoticeReceiveList(receiveList);
        }

        // 保存消息到ES
        if (notice != null) {
            this.getDictMap();
            notice.setNoticeTypeName(dictMap.get(notice.getNoticeType()));
            notice.setNoticeSourceName(dictMap.get(notice.getNoticeSource()));
        }

        this.saveHistoryData(notice);
    }

    /**
     * 消息推送
     * @param notice
     */
    @Transactional(readOnly = false)
    public void publish(Message notice) {
        Integer noticeId = notice.getId();
        // 极光推送到手机端
        // 获取接收人信息
        List<String> receiveIds = new ArrayList<>();
        MessageReceive receive = new MessageReceive();
        receive.setNoticeId(noticeId);
        List<MessageReceive> receiveList = messageReceiveService.findList(receive);
        if (ListUtil.isNotEmpty(receiveList)) {
            receiveIds = receiveList.stream().map(p->p.getReceiveId()).collect(Collectors.toList());
        }
        notice = dao.get(noticeId);
        String allFlag = notice.getAllFlag();
        Map<String, String> pushMap = new HashMap<>();
        pushMap.put("message", JSONUtils.obj2json(notice));
        if ("0".equals(allFlag)) {
            // 推送到手机端，指定用户
            pushMap.put("userIds", StringUtils.join(receiveIds, ","));
            try {
                jPushApiService.pushAndroid(pushMap);
            } catch (Exception e){
                logger.error("极光推送异常：{}", e.getMessage());
            }
        }

        // 推送到PC端
        List<Message> list = dao.findListByReceiveIds(receiveIds);
        if (ListUtil.isNotEmpty(list) && ListUtil.isNotEmpty(receiveIds)) {
            this.assemblyNoticeData(list);
            Map<String, List<Message>> map = list.stream().collect(Collectors.groupingBy(Message::getReceiveId));
            Map<String, String> sendMap = new HashMap<>();
            for (String receiveId : receiveIds) {
                if (map.containsKey(receiveId)) {
                    // 最多只推送15条
                    List<Message> tempList = map.get(receiveId);
                    Map<String, Object> receiveMap = new HashMap<>();
                    int size = tempList.size();
                    receiveMap.put("count", size);
                    if (size > 15) {
                        tempList = tempList.subList(0, 15);
                    }
                    receiveMap.put("list", tempList);
                    // 服务端发送消息给指定客户
                    sendMap.put(receiveId, JSONUtils.obj2json(receiveMap));
                }
            }
            //webSocketApiService.sendMessageToUsers(sendMap);
            try {
                this.sendMessage(sendMap);
            } catch (Exception e){
                logger.error("websocket推送异常：{}", e.getMessage());
            }
        }
    }

    /**
     * 推送消息
     * @param sendMap
     */
    private void sendMessage(Map<String, String> sendMap) {
        String serverInfo = this.getWebsocketServer();
        if (!StringUtils.isNotEmpty(serverInfo)) {
            return;
        }
        try {
            WebSocketClient webSocketClient = new WebSocketClient();
            JSONArray jsonArray = JSON.parseArray(serverInfo);
            for(int i = 0; i < jsonArray.size(); i++) {
                JSONObject jsonObject = (JSONObject) jsonArray.get(i);
                String userIdsStr = jsonObject.getString("userIds");
                if (StringUtils.isNotEmpty(userIdsStr)) {
                    String[] userIds = userIdsStr.split(",");
                    String address = jsonObject.getString("address");
                    String uri = "ws://" + address + "/websocket?userId=webSocketClient";
                    webSocketClient.start(uri);
                    if (webSocketClient.session == null) {
                        continue;
                    }
                    JSONArray sendArray = new JSONArray();
                    for (String userId : userIds) {
                        if (sendMap.containsKey(userId)) {
                            JSONObject object = new JSONObject();
                            object.put("userId", userId);
                            object.put("msg", sendMap.get(userId));
                            sendArray.add(object);
                        }
                    }
                    webSocketClient.session.getBasicRemote().sendText(sendArray.toString());
                    webSocketClient.session.close();
                }
             }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 获取用户连接的websocket服务器信息
     * @return
     */
    private String getWebsocketServer() {
        String serverInfo = null;
        AjaxJson json = webSocketApiService.getWebsocketServer();
        if (json.isSuccess()) {
            Object obj = json.getBody();
            if (obj != null) {
                serverInfo = (String) ((LinkedHashMap) obj).get("serverInfo");
            }
        }
        return serverInfo;
    }

    /**
     * 消息删除
     * @param ids
     */
    @Transactional(readOnly = false)
    public void deleteAll(String ids) {
        if (StringUtils.isNotEmpty(ids)) {
            List<Long> noticeIds = new ArrayList<>();
            String[] str = ids.split(",");
            for (String id : str) {
                noticeIds.add(Long.valueOf(id));
            }
            messageReceiveService.deleteByNoticeIds(noticeIds);
            dao.deleteByIds(noticeIds);
        }
    }
}